/*
 * =====================================================================================
 *
 *       Filename:  bch.h
 *
 *    Description:  systematic triple-error-correcting BCH encoder and decoder declarations
 *
 *        Version:  1.0
 *        Created:  2020-06-02 03:40:39 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Alvin Sukmadji 
 *   Organization:  University of Toronto
 *   
 *   Note: This license is a modified BSD License, which contains the terms of noncommercial
 *   use.
 *   
 *   Copyright <2020> <Alvin Yonathan Sukmadji and Frank Robert Kschischang>
 *   
 *   Redistribution and use in source and binary forms, with or without modification, are
 *   permitted provided that the following conditions are met:
 *   
 *   1. Redistributions of source code must retain the above copyright notice, this list of
 *   conditions and the following disclaimer.
 *   
 *   2. Redistributions in binary form must reproduce the above copyright notice, this list
 *   of conditions and the following disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 *   
 *   3. Neither the name of the copyright holder nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific prior written
 *   permission.
 *   
 *   4. This software may only be used for noncommercial purposes, including teaching and
 *   academic research at universities, colleges and other educational institutions, and
 *   personal non-profit purposes. Neither this software, whether in Source or Object form,
 *   nor any derivative work that is based on (or derived from) this software may be used for
 *   commercial purposes, for example, licensing or selling the software, or using the
 *   software with a purpose to procure a commercial gain.
 *   
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * =====================================================================================
 */

#ifndef BCH_H
#define BCH_H

// fill buffer with random 0/1
void fillrand(unsigned char *buffer, int len, pcg32f_random_t *rng);

// prints BCH codeword, for debugging
void BCH_print(unsigned char *x, int n);

// long division of x with genpoly (see bch.c) over GF(2)
void long_division(unsigned char *x, unsigned char *rem, int n, int k);

// computing parity bits of x array, uses long division
// writes the remainder to rem array
void compute_parity(unsigned char *x, unsigned char *rem, int n, int k);

// BCH (n,k) encoder implementation
void BCH_encode(unsigned char *x, int n, int k);

// compute the syndrome of a BCH codeword, n<=4095
unsigned long long int BCH_compute_syndrome(unsigned char *x, int n);

// compute the syndrome of a BCH codeword with 1 at position pos and 0 everywhere else, n<=4095
unsigned long long int BCH_compute_syndrome_pos(unsigned int pos, int n);

// bch decoder implementation
int BCH_decode(unsigned long long int syndrome,unsigned int *p1, unsigned int *p2, unsigned int *p3);

// get BCH code length
int BCH_get_n();

// get BCH code dimension
int BCH_get_k();

#endif //BCH_H