/*
 * =====================================================================================
 *
 *       Filename:  common.h
 *
 *    Description:  declaration of the structures for zipper codes
 *
 *        Version:  1.0
 *        Created:  2020-11-29 03:40:39 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Alvin Sukmadji 
 *   Organization:  University of Toronto
 *   
 *   Note: This license is a modified BSD License, which contains the terms of noncommercial
 *   use.
 *   
 *   Copyright <2020> <Alvin Yonathan Sukmadji>
 *   
 *   Redistribution and use in source and binary forms, with or without modification, are
 *   permitted provided that the following conditions are met:
 *   
 *   1. Redistributions of source code must retain the above copyright notice, this list of
 *   conditions and the following disclaimer.
 *   
 *   2. Redistributions in binary form must reproduce the above copyright notice, this list
 *   of conditions and the following disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 *   
 *   3. Neither the name of the copyright holder nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific prior written
 *   permission.
 *   
 *   4. This software may only be used for noncommercial purposes, including teaching and
 *   academic research at universities, colleges and other educational institutions, and
 *   personal non-profit purposes. Neither this software, whether in Source or Object form,
 *   nor any derivative work that is based on (or derived from) this software may be used for
 *   commercial purposes, for example, licensing or selling the software, or using the
 *   software with a purpose to procure a commercial gain.
 *   
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * =====================================================================================
 */

#ifndef COMMON_H
#define COMMON_H

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include <stdbool.h>
#include <unistd.h>

typedef struct interleaver_map
{
    int row_diff;
    int col;
}I_MAP;

typedef struct zipper
{
    int period;
    int *n;
    int *k;
    int *m;

    int max_lookback;
    int n_max;
    int buffer_len;
    int extended_buffer_len;
    int chunk_size;

    I_MAP ***i_map; //interleaver map
    int **num_copy;

    unsigned char **buffer_sent; //encoder-side buffer
    unsigned char **buffer_recv; //decoder-side buffer
    unsigned long long int *syndromes;

    bool *is_fresh;
}ZIPPER;
#endif //COMMON_H